@@ -19,17 +19,18 @@ import (
 	"golang.org/x/text/encoding/unicode"
 )
 
-var imageFormat decode.Group
+var imageGroup decode.Group
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:        format.ID3V2,
-		Description: "ID3v2 metadata",
-		DecodeFn:    id3v2Decode,
-		Dependencies: []decode.Dependency{
-			{Names: []string{format.IMAGE}, Group: &imageFormat},
-		},
-	})
+	interp.RegisterFormat(
+		format.ID3v2,
+		&decode.Format{
+			Description: "ID3v2 metadata",
+			DecodeFn:    id3v2Decode,
+			Dependencies: []decode.Dependency{
+				{Groups: []*decode.Group{format.Image}, Out: &imageGroup},
+			},
+		})
 }
 
 var idDescriptions = scalar.StrMapDescription{
@@ -424,9 +425,23 @@ func decodeFrame(d *decode.D, version int) uint64 {
 			d.FieldU32("end_offset")
 			decodeFrames(d, version, uint64(d.BitsLeft()/8))
 		},
+
+		// <ID3v2.3 or ID3v2.4 frame header, ID: "CTOC">   (10 bytes)
+		// Element ID      <text string> $00
+		// Flags           %000000ab
+		// Entry count     $xx  (8-bit unsigned int)
+		// <Child Element ID list>
+		// <Optional embedded sub-frames>
+		//
+		// flag a: Top-level bit
+		// flag b: Ordered bit
 		"CTOC": func(d *decode.D) {
 			d.FieldStrFn("element_id", textNullFn(encodingUTF8))
-			d.FieldU8("ctoc_flags")
+			d.FieldStruct("ctoc_flags", func(d *decode.D) {
+				d.FieldU6("unused0")
+				d.FieldBool("top_level")
+				d.FieldBool("order")
+			})
 			entryCount := d.FieldU8("entry_count")
 			d.FieldArray("entries", func(d *decode.D) {
 				for i := uint64(0); i < entryCount; i++ {
@@ -449,7 +464,7 @@ func decodeFrame(d *decode.D, version int) uint64 {
 			d.FieldUTF8("image_format", 3)
 			d.FieldU8("picture_type") // TODO: table
 			d.FieldStrFn("description", textNullFn(int(encoding)))
-			d.FieldFormatOrRawLen("picture", d.BitsLeft(), imageFormat, nil)
+			d.FieldFormatOrRawLen("picture", d.BitsLeft(), &imageGroup, nil)
 		},
 
 		// <Header for 'Attached picture', ID: "APIC">
@@ -463,7 +478,7 @@ func decodeFrame(d *decode.D, version int) uint64 {
 			d.FieldStrFn("mime_type", textNullFn(encodingUTF8))
 			d.FieldU8("picture_type") // TODO: table
 			d.FieldStrFn("description", textNullFn(int(encoding)))
-			d.FieldFormatOrRawLen("picture", d.BitsLeft(), imageFormat, nil)
+			d.FieldFormatOrRawLen("picture", d.BitsLeft(), &imageGroup, nil)
 		},
 
 		// <Header for 'General encapsulated object', ID: "GEOB">
@@ -477,7 +492,7 @@ func decodeFrame(d *decode.D, version int) uint64 {
 			d.FieldStrFn("mime_type", textNullFn(encodingUTF8))
 			d.FieldStrFn("filename", textNullFn(int(encoding)))
 			d.FieldStrFn("description", textNullFn(int(encoding)))
-			d.FieldFormatOrRawLen("data", d.BitsLeft(), imageFormat, nil)
+			d.FieldFormatOrRawLen("data", d.BitsLeft(), &imageGroup, nil)
 		},
 
 		// Unsynced lyrics/text "ULT"
@@ -526,6 +541,7 @@ func decodeFrame(d *decode.D, version int) uint64 {
 			encoding := d.FieldU8("text_encoding", encodingNames)
 			d.FieldStrFn("text", textFn(int(encoding), int(d.BitsLeft()/8)))
 		},
+
 		// User defined...   "TXX"
 		// Frame size        $xx xx xx
 		// Text encoding     $xx
@@ -542,6 +558,23 @@ func decodeFrame(d *decode.D, version int) uint64 {
 			d.FieldStrFn("description", textNullLenFn(int(encoding), int(d.BitsLeft()/8)))
 			d.FieldStrFn("value", textFn(int(encoding), int(d.BitsLeft()/8)))
 		},
+
+		// User defined...   "WXX"
+		// Frame size        $xx xx xx
+		// Text encoding     $xx
+		// Description       <textstring> $00 (00)
+		// URL               <textstring>
+		//
+		// <Header for 'User defined URL link frame', ID: "WXXX">
+		// Text encoding     $xx
+		// Description       <text string according to encoding> $00 (00)
+		// URL               <text string>
+		"WXXX": func(d *decode.D) {
+			encoding := d.FieldU8("text_encoding", encodingNames)
+			d.FieldStrFn("description", textNullLenFn(int(encoding), int(d.BitsLeft()/8)))
+			d.FieldStrFn("url", textFn(int(encoding), int(d.BitsLeft()/8)))
+		},
+
 		// <Header for 'Private frame', ID: "PRIV">
 		// Owner identifier      <text string> $00
 		// The private data      <binary data>
@@ -558,6 +591,8 @@ func decodeFrame(d *decode.D, version int) uint64 {
 		idNormalized = "COMM"
 	case id == "TXX", id == "TXXX":
 		idNormalized = "TXXX"
+	case id == "WXX", id == "WXXX":
+		idNormalized = "WXXX"
 	case len(id) > 0 && id[0] == 'T':
 		idNormalized = "T000"
 	}
