@@ -10,19 +10,20 @@ import (
 var udpPayloadGroup decode.Group
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:        format.UDP_DATAGRAM,
-		Description: "User datagram protocol",
-		Groups:      []string{format.IP_PACKET},
-		Dependencies: []decode.Dependency{
-			{Names: []string{format.UDP_PAYLOAD}, Group: &udpPayloadGroup},
-		},
-		DecodeFn: decodeUDP,
-	})
+	interp.RegisterFormat(
+		format.UDP_Datagram,
+		&decode.Format{
+			Description: "User datagram protocol",
+			Groups:      []*decode.Group{format.IP_Packet},
+			Dependencies: []decode.Dependency{
+				{Groups: []*decode.Group{format.UDP_Payload}, Out: &udpPayloadGroup},
+			},
+			DecodeFn: decodeUDP,
+		})
 }
 
 func decodeUDP(d *decode.D) any {
-	var ipi format.IPPacketIn
+	var ipi format.IP_Packet_In
 	if d.ArgAs(&ipi) && ipi.Protocol != format.IPv4ProtocolUDP {
 		d.Fatalf("incorrect protocol %d", ipi.Protocol)
 	}
@@ -36,8 +37,8 @@ func decodeUDP(d *decode.D) any {
 	d.FieldFormatOrRawLen(
 		"payload",
 		payloadLen,
-		udpPayloadGroup,
-		format.UDPPayloadIn{
+		&udpPayloadGroup,
+		format.UDP_Payload_In{
 			SourcePort:      int(sourcePort),
 			DestinationPort: int(destPort),
 		},
