@@ -39,19 +39,20 @@ import (
 //go:embed tls.md
 var tlsFS embed.FS
 
-var asn1BerFormat decode.Group
+var asn1BerGroup decode.Group
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:         format.TLS,
-		Description:  "Transport layer security",
-		Groups:       []string{format.TCP_STREAM},
-		DecodeFn:     decodeTLS,
-		DefaultInArg: format.TLSIn{},
-		Dependencies: []decode.Dependency{
-			{Names: []string{format.ASN1_BER}, Group: &asn1BerFormat},
-		},
-	})
+	interp.RegisterFormat(
+		format.TLS,
+		&decode.Format{
+			Description:  "Transport layer security",
+			Groups:       []*decode.Group{format.TCP_Stream},
+			DecodeFn:     decodeTLS,
+			DefaultInArg: format.TLS_In{},
+			Dependencies: []decode.Dependency{
+				{Groups: []*decode.Group{format.ASN1_BER}, Out: &asn1BerGroup},
+			},
+		})
 	interp.RegisterFS(tlsFS)
 }
 
@@ -366,7 +367,7 @@ func decodeTLSHandshake(d *decode.D, tc *tlsCtx) {
 					for !d.End() {
 						d.FieldStruct("certificate", func(d *decode.D) {
 							length := d.FieldU24("length")
-							d.FieldFormatLen("data", int64(length)*8, asn1BerFormat, nil)
+							d.FieldFormatLen("data", int64(length)*8, &asn1BerGroup, nil)
 						})
 					}
 				})
@@ -675,12 +676,12 @@ func decodeV2ClientHello(d *decode.D, tc *tlsCtx) {
 }
 
 func decodeTLS(d *decode.D) any {
-	var ti format.TLSIn
+	var ti format.TLS_In
 	d.ArgAs(&ti)
 
 	isClient := false
 
-	var tsi format.TCPStreamIn
+	var tsi format.TCP_Stream_In
 	if d.ArgAs(&tsi) {
 		if !tsi.HasStart {
 			d.Fatalf("tls requires start of byte stream")
@@ -716,10 +717,10 @@ func decodeTLS(d *decode.D) any {
 
 	// client side will do post for both
 	if !isClient {
-		return format.TCPStreamOut{InArg: tc}
+		return format.TCP_Stream_Out{InArg: tc}
 	}
 
-	return format.TCPStreamOut{
+	return format.TCP_Stream_Out{
 		PostFn: func(peerIn any) {
 			// peerIn will be the other peers outArg, the server *tlsCtx
 			clientTc := tc
