@@ -36,19 +36,20 @@ import (
 var xmlFS embed.FS
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:        format.XML,
-		Description: "Extensible Markup Language",
-		ProbeOrder:  format.ProbeOrderTextFuzzy,
-		Groups:      []string{format.PROBE},
-		DecodeFn:    decodeXML,
-		DefaultInArg: format.XMLIn{
-			Seq:             false,
-			Array:           false,
-			AttributePrefix: "@",
-		},
-		Functions: []string{"_todisplay"},
-	})
+	interp.RegisterFormat(
+		format.XML,
+		&decode.Format{
+			Description: "Extensible Markup Language",
+			ProbeOrder:  format.ProbeOrderTextFuzzy,
+			Groups:      []*decode.Group{format.Probe},
+			DecodeFn:    decodeXML,
+			DefaultInArg: format.XML_In{
+				Seq:             false,
+				Array:           false,
+				AttributePrefix: "@",
+			},
+			Functions: []string{"_todisplay"},
+		})
 	interp.RegisterFS(xmlFS)
 	interp.RegisterFunc1("to_xml", toXML)
 	interp.RegisterFunc0("from_xmlentities", func(_ *interp.Interp, c string) any {
@@ -115,7 +116,7 @@ func elmName(space, local string) string {
 	return space + ":" + local
 }
 
-func fromXMLToObject(n xmlNode, xi format.XMLIn) any {
+func fromXMLToObject(n xmlNode, xi format.XML_In) any {
 	var f func(n xmlNode, seq int, nss xmlNNStack) (string, any)
 	f = func(n xmlNode, seq int, nss xmlNNStack) (string, any) {
 		attrs := map[string]any{}
@@ -282,7 +283,7 @@ func decodeXMLSeekFirstValidRune(br io.ReadSeeker) error {
 }
 
 func decodeXML(d *decode.D) any {
-	var xi format.XMLIn
+	var xi format.XML_In
 	d.ArgAs(&xi)
 
 	bbr := d.RawLen(d.Len())
