@@ -29,23 +29,23 @@ const (
 )
 
 type Options struct {
-	Name          string
-	Description   string
-	Force         bool
-	FillGaps      bool
-	IsRoot        bool
-	Range         ranges.Range // if zero use whole buffer
-	InArg         any
-	FormatInArgFn func(init any) any
-	ReadBuf       *[]byte
+	Name        string
+	Description string
+	Force       bool
+	FillGaps    bool
+	IsRoot      bool
+	Range       ranges.Range // if zero use whole buffer
+	InArg       any
+	ParseOptsFn func(init any) any
+	ReadBuf     *[]byte
 }
 
 // Decode try decode group and return first success and all other decoder errors
-func Decode(ctx context.Context, br bitio.ReaderAtSeeker, group Group, opts Options) (*Value, any, error) {
+func Decode(ctx context.Context, br bitio.ReaderAtSeeker, group *Group, opts Options) (*Value, any, error) {
 	return decode(ctx, br, group, opts)
 }
 
-func decode(ctx context.Context, br bitio.ReaderAtSeeker, group Group, opts Options) (*Value, any, error) {
+func decode(ctx context.Context, br bitio.ReaderAtSeeker, group *Group, opts Options) (*Value, any, error) {
 	brLen, err := bitioex.Len(br)
 	if err != nil {
 		return nil, nil, err
@@ -60,23 +60,31 @@ func decode(ctx context.Context, br bitio.ReaderAtSeeker, group Group, opts Opti
 		panic("group is nil, failed to register format?")
 	}
 
+	hasGroupOpts := false
+	groupArg := group.DefaultInArg
+	if opts.ParseOptsFn != nil && groupArg != nil {
+		if groupOptArg := opts.ParseOptsFn(groupArg); groupOptArg != nil {
+			hasGroupOpts = true
+			groupArg = groupOptArg
+		}
+	}
+
 	formatsErr := FormatsError{}
 
-	for _, f := range group {
+	for _, f := range group.Formats {
 		var inArgs []any
 
 		// figure out if there are format specific arg passed as options
 		hasFormatOpts := false
 		formatArg := f.DefaultInArg
-		if formatArg != nil && opts.FormatInArgFn != nil {
-			formatOptArg := opts.FormatInArgFn(formatArg)
-			if formatOptArg != nil {
+		if opts.ParseOptsFn != nil && formatArg != nil {
+			if formatOptArg := opts.ParseOptsFn(formatArg); formatOptArg != nil {
 				hasFormatOpts = true
 				formatArg = formatOptArg
 			}
 		}
 
-		// format options have prio
+		// format options have priority
 		if hasFormatOpts {
 			inArgs = append(inArgs, formatArg)
 		}
@@ -87,6 +95,10 @@ func decode(ctx context.Context, br bitio.ReaderAtSeeker, group Group, opts Opti
 			inArgs = append(inArgs, f.DefaultInArg)
 		}
 
+		if hasGroupOpts {
+			inArgs = append(inArgs, groupArg)
+		}
+
 		cBR, err := bitioex.Range(br, decodeRange.Start, decodeRange.Len)
 		if err != nil {
 			return nil, nil, IOError{Err: err, Op: "BitBufRange", ReadSize: decodeRange.Len, Pos: decodeRange.Start}
@@ -127,7 +139,7 @@ func decode(ctx context.Context, br bitio.ReaderAtSeeker, group Group, opts Opti
 				d.Value.Err = formatErr
 			}
 
-			if len(group) != 1 {
+			if len(group.Formats) != 1 {
 				continue
 			}
 		}
@@ -178,7 +190,7 @@ type D struct {
 
 // TODO: new struct decoder?
 // note br is assumed to be a non-shared buffer
-func newDecoder(ctx context.Context, format Format, br bitio.ReaderAtSeeker, opts Options) *D {
+func newDecoder(ctx context.Context, format *Format, br bitio.ReaderAtSeeker, opts Options) *D {
 	name := format.RootName
 	if opts.Name != "" {
 		name = opts.Name
@@ -198,7 +210,7 @@ func newDecoder(ctx context.Context, format Format, br bitio.ReaderAtSeeker, opt
 			RootReader: br,
 			Range:      ranges.Range{Start: 0, Len: 0},
 			IsRoot:     opts.IsRoot,
-			Format:     &format,
+			Format:     format,
 		},
 		Options: opts,
 
@@ -818,7 +830,6 @@ func (d *D) FieldGet(name string) *Value {
 	}
 	return nil
 }
-
 func (d *D) FieldMustGet(name string) *Value {
 	if v := d.FieldGet(name); v != nil {
 		return v
@@ -889,6 +900,12 @@ func (d *D) FieldRangeFn(name string, firstBit int64, nBits int64, fn func() *Va
 	return v
 }
 
+func (d *D) AssertPos(pos int64) {
+	if d.Pos() != pos {
+		panic(DecoderError{Reason: fmt.Sprintf("expected bits position %d", pos), Pos: d.Pos()})
+	}
+}
+
 func (d *D) AssertAtLeastBitsLeft(nBits int64) {
 	if d.Options.Force {
 		return
@@ -951,15 +968,15 @@ func (d *D) RangeFn(firstBit int64, nBits int64, fn func(d *D)) int64 {
 	return endPos - startPos
 }
 
-func (d *D) Format(group Group, inArg any) any {
+func (d *D) Format(group *Group, inArg any) any {
 	dv, v, err := decode(d.Ctx, d.bitBuf, group, Options{
-		Force:         d.Options.Force,
-		FillGaps:      false,
-		IsRoot:        false,
-		Range:         ranges.Range{Start: d.Pos(), Len: d.BitsLeft()},
-		InArg:         inArg,
-		FormatInArgFn: d.Options.FormatInArgFn,
-		ReadBuf:       d.readBuf,
+		Force:       d.Options.Force,
+		FillGaps:    false,
+		IsRoot:      false,
+		Range:       ranges.Range{Start: d.Pos(), Len: d.BitsLeft()},
+		InArg:       inArg,
+		ParseOptsFn: d.Options.ParseOptsFn,
+		ReadBuf:     d.readBuf,
 	})
 	if dv == nil || dv.Errors() != nil {
 		d.IOPanic(err, "Format: decode")
@@ -981,16 +998,16 @@ func (d *D) Format(group Group, inArg any) any {
 	return v
 }
 
-func (d *D) TryFieldFormat(name string, group Group, inArg any) (*Value, any, error) {
+func (d *D) TryFieldFormat(name string, group *Group, inArg any) (*Value, any, error) {
 	dv, v, err := decode(d.Ctx, d.bitBuf, group, Options{
-		Name:          name,
-		Force:         d.Options.Force,
-		FillGaps:      false,
-		IsRoot:        false,
-		Range:         ranges.Range{Start: d.Pos(), Len: d.BitsLeft()},
-		InArg:         inArg,
-		FormatInArgFn: d.Options.FormatInArgFn,
-		ReadBuf:       d.readBuf,
+		Name:        name,
+		Force:       d.Options.Force,
+		FillGaps:    false,
+		IsRoot:      false,
+		Range:       ranges.Range{Start: d.Pos(), Len: d.BitsLeft()},
+		InArg:       inArg,
+		ParseOptsFn: d.Options.ParseOptsFn,
+		ReadBuf:     d.readBuf,
 	})
 	if dv == nil || dv.Errors() != nil {
 		return nil, nil, err
@@ -1004,7 +1021,7 @@ func (d *D) TryFieldFormat(name string, group Group, inArg any) (*Value, any, er
 	return dv, v, err
 }
 
-func (d *D) FieldFormat(name string, group Group, inArg any) (*Value, any) {
+func (d *D) FieldFormat(name string, group *Group, inArg any) (*Value, any) {
 	dv, v, err := d.TryFieldFormat(name, group, inArg)
 	if dv == nil || dv.Errors() != nil {
 		d.IOPanic(err, "FieldFormat: TryFieldFormat")
@@ -1012,7 +1029,7 @@ func (d *D) FieldFormat(name string, group Group, inArg any) (*Value, any) {
 	return dv, v
 }
 
-func (d *D) FieldFormatOrRaw(name string, group Group, inArg any) (*Value, any) {
+func (d *D) FieldFormatOrRaw(name string, group *Group, inArg any) (*Value, any) {
 	dv, v, _ := d.TryFieldFormat(name, group, inArg)
 	if dv == nil {
 		d.FieldRawLen(name, d.BitsLeft())
@@ -1020,16 +1037,16 @@ func (d *D) FieldFormatOrRaw(name string, group Group, inArg any) (*Value, any)
 	return dv, v
 }
 
-func (d *D) TryFieldFormatLen(name string, nBits int64, group Group, inArg any) (*Value, any, error) {
+func (d *D) TryFieldFormatLen(name string, nBits int64, group *Group, inArg any) (*Value, any, error) {
 	dv, v, err := decode(d.Ctx, d.bitBuf, group, Options{
-		Name:          name,
-		Force:         d.Options.Force,
-		FillGaps:      true,
-		IsRoot:        false,
-		Range:         ranges.Range{Start: d.Pos(), Len: nBits},
-		InArg:         inArg,
-		FormatInArgFn: d.Options.FormatInArgFn,
-		ReadBuf:       d.readBuf,
+		Name:        name,
+		Force:       d.Options.Force,
+		FillGaps:    true,
+		IsRoot:      false,
+		Range:       ranges.Range{Start: d.Pos(), Len: nBits},
+		InArg:       inArg,
+		ParseOptsFn: d.Options.ParseOptsFn,
+		ReadBuf:     d.readBuf,
 	})
 	if dv == nil || dv.Errors() != nil {
 		return nil, nil, err
@@ -1043,7 +1060,7 @@ func (d *D) TryFieldFormatLen(name string, nBits int64, group Group, inArg any)
 	return dv, v, err
 }
 
-func (d *D) FieldFormatLen(name string, nBits int64, group Group, inArg any) (*Value, any) {
+func (d *D) FieldFormatLen(name string, nBits int64, group *Group, inArg any) (*Value, any) {
 	dv, v, err := d.TryFieldFormatLen(name, nBits, group, inArg)
 	if dv == nil || dv.Errors() != nil {
 		d.IOPanic(err, "FieldFormatLen: TryFieldFormatLen")
@@ -1051,7 +1068,7 @@ func (d *D) FieldFormatLen(name string, nBits int64, group Group, inArg any) (*V
 	return dv, v
 }
 
-func (d *D) FieldFormatOrRawLen(name string, nBits int64, group Group, inArg any) (*Value, any) {
+func (d *D) FieldFormatOrRawLen(name string, nBits int64, group *Group, inArg any) (*Value, any) {
 	dv, v, _ := d.TryFieldFormatLen(name, nBits, group, inArg)
 	if dv == nil {
 		d.FieldRawLen(name, nBits)
@@ -1060,16 +1077,16 @@ func (d *D) FieldFormatOrRawLen(name string, nBits int64, group Group, inArg any
 }
 
 // TODO: return decooder?
-func (d *D) TryFieldFormatRange(name string, firstBit int64, nBits int64, group Group, inArg any) (*Value, any, error) {
+func (d *D) TryFieldFormatRange(name string, firstBit int64, nBits int64, group *Group, inArg any) (*Value, any, error) {
 	dv, v, err := decode(d.Ctx, d.bitBuf, group, Options{
-		Name:          name,
-		Force:         d.Options.Force,
-		FillGaps:      true,
-		IsRoot:        false,
-		Range:         ranges.Range{Start: firstBit, Len: nBits},
-		InArg:         inArg,
-		FormatInArgFn: d.Options.FormatInArgFn,
-		ReadBuf:       d.readBuf,
+		Name:        name,
+		Force:       d.Options.Force,
+		FillGaps:    true,
+		IsRoot:      false,
+		Range:       ranges.Range{Start: firstBit, Len: nBits},
+		InArg:       inArg,
+		ParseOptsFn: d.Options.ParseOptsFn,
+		ReadBuf:     d.readBuf,
 	})
 	if dv == nil || dv.Errors() != nil {
 		return nil, nil, err
@@ -1080,7 +1097,7 @@ func (d *D) TryFieldFormatRange(name string, firstBit int64, nBits int64, group
 	return dv, v, err
 }
 
-func (d *D) FieldFormatRange(name string, firstBit int64, nBits int64, group Group, inArg any) (*Value, any) {
+func (d *D) FieldFormatRange(name string, firstBit int64, nBits int64, group *Group, inArg any) (*Value, any) {
 	dv, v, err := d.TryFieldFormatRange(name, firstBit, nBits, group, inArg)
 	if dv == nil || dv.Errors() != nil {
 		d.IOPanic(err, "FieldFormatRange: TryFieldFormatRange")
@@ -1089,15 +1106,15 @@ func (d *D) FieldFormatRange(name string, firstBit int64, nBits int64, group Gro
 	return dv, v
 }
 
-func (d *D) TryFieldFormatBitBuf(name string, br bitio.ReaderAtSeeker, group Group, inArg any) (*Value, any, error) {
+func (d *D) TryFieldFormatBitBuf(name string, br bitio.ReaderAtSeeker, group *Group, inArg any) (*Value, any, error) {
 	dv, v, err := decode(d.Ctx, br, group, Options{
-		Name:          name,
-		Force:         d.Options.Force,
-		FillGaps:      true,
-		IsRoot:        true,
-		InArg:         inArg,
-		FormatInArgFn: d.Options.FormatInArgFn,
-		ReadBuf:       d.readBuf,
+		Name:        name,
+		Force:       d.Options.Force,
+		FillGaps:    true,
+		IsRoot:      true,
+		InArg:       inArg,
+		ParseOptsFn: d.Options.ParseOptsFn,
+		ReadBuf:     d.readBuf,
 	})
 	if dv == nil || dv.Errors() != nil {
 		return nil, nil, err
@@ -1110,7 +1127,7 @@ func (d *D) TryFieldFormatBitBuf(name string, br bitio.ReaderAtSeeker, group Gro
 	return dv, v, err
 }
 
-func (d *D) FieldFormatBitBuf(name string, br bitio.ReaderAtSeeker, group Group, inArg any) (*Value, any) {
+func (d *D) FieldFormatBitBuf(name string, br bitio.ReaderAtSeeker, group *Group, inArg any) (*Value, any) {
 	dv, v, err := d.TryFieldFormatBitBuf(name, br, group, inArg)
 	if dv == nil || dv.Errors() != nil {
 		d.IOPanic(err, "FieldFormatBitBuf: TryFieldFormatBitBuf")
@@ -1168,7 +1185,7 @@ func (d *D) FieldStructRootBitBufFn(name string, br bitio.ReaderAtSeeker, fn fun
 }
 
 // TODO: range?
-func (d *D) FieldFormatReaderLen(name string, nBits int64, fn func(r io.Reader) (io.ReadCloser, error), group Group) (*Value, any) {
+func (d *D) FieldFormatReaderLen(name string, nBits int64, fn func(r io.Reader) (io.ReadCloser, error), group *Group) (*Value, any) {
 	br, err := d.TryBitBufLen(nBits)
 	if err != nil {
 		d.IOPanic(err, "FieldFormatReaderLen: BitBufLen")
@@ -1189,7 +1206,7 @@ func (d *D) FieldFormatReaderLen(name string, nBits int64, fn func(r io.Reader)
 }
 
 // TODO: too mant return values
-func (d *D) TryFieldReaderRangeFormat(name string, startBit int64, nBits int64, fn func(r io.Reader) io.Reader, group Group, inArg any) (int64, bitio.ReaderAtSeeker, *Value, any, error) {
+func (d *D) TryFieldReaderRangeFormat(name string, startBit int64, nBits int64, fn func(r io.Reader) io.Reader, group *Group, inArg any) (int64, bitio.ReaderAtSeeker, *Value, any, error) {
 	bitLen := nBits
 	if bitLen == -1 {
 		bitLen = d.BitsLeft()
@@ -1213,7 +1230,7 @@ func (d *D) TryFieldReaderRangeFormat(name string, startBit int64, nBits int64,
 	return cz * 8, rbr, dv, v, err
 }
 
-func (d *D) FieldReaderRangeFormat(name string, startBit int64, nBits int64, fn func(r io.Reader) io.Reader, group Group, inArg any) (int64, bitio.ReaderAtSeeker, *Value, any) {
+func (d *D) FieldReaderRangeFormat(name string, startBit int64, nBits int64, fn func(r io.Reader) io.Reader, group *Group, inArg any) (int64, bitio.ReaderAtSeeker, *Value, any) {
 	cz, rBR, dv, v, err := d.TryFieldReaderRangeFormat(name, startBit, nBits, fn, group, inArg)
 	if err != nil {
 		d.IOPanic(err, "TryFieldReaderRangeFormat")
@@ -1244,15 +1261,11 @@ func (d *D) FieldValue(name string, fn func() *Value) *Value {
 	return v
 }
 
-func (d *D) RE(reRef **regexp.Regexp, reStr string) []ranges.Range {
-	if *reRef == nil {
-		*reRef = regexp.MustCompile(reStr)
-	}
-
+func (d *D) RE(re *regexp.Regexp) []ranges.Range {
 	startPos := d.Pos()
 
 	rr := ioex.ByteRuneReader{RS: bitio.NewIOReadSeeker(d.bitBuf)}
-	locs := (*reRef).FindReaderSubmatchIndex(rr)
+	locs := re.FindReaderSubmatchIndex(rr)
 	if locs == nil {
 		return nil
 	}
@@ -1275,13 +1288,10 @@ func (d *D) RE(reRef **regexp.Regexp, reStr string) []ranges.Range {
 	return rs
 }
 
-func (d *D) FieldRE(reRef **regexp.Regexp, reStr string, mRef *map[string]string, sms ...scalar.StrMapper) {
-	if *reRef == nil {
-		*reRef = regexp.MustCompile(reStr)
-	}
-	subexpNames := (*reRef).SubexpNames()
+func (d *D) FieldRE(re *regexp.Regexp, mRef *map[string]string, sms ...scalar.StrMapper) {
+	subexpNames := re.SubexpNames()
 
-	rs := d.RE(reRef, reStr)
+	rs := d.RE(re)
 	for i, r := range rs {
 		if i == 0 || r.Start == -1 {
 			continue
